import pytest
import os
from tempfile import TemporaryDirectory
from erdgen.src.erdgen import (
    load_yml,
    generate_dbml,
    generate_dbml_schema,
    extract_relationships,
    extract_columns,
)

sample_yaml_content = """
version: 2
models:
 - name: Computer
   description: beep boop beep
   columns:
     - name: computerId
       description: The unique identifier of computer
   relationships:
     - name: files
       description: The files are in the computer!?
       type: one_to_many
       table: computer_files
       join:
         - local: computerId
           remote: computerId
"""


def test_load_yml():
    with TemporaryDirectory() as tempdir:
        test_file_path = os.path.join(tempdir, "model.yml")
        with open(test_file_path, "w") as f:
            f.write(sample_yaml_content)
        parsed = load_yml(test_file_path)
        assert isinstance(parsed, dict)
        assert parsed["models"][0]["name"] == "Computer"
        assert len(parsed["models"][0]["relationships"]) == 1


def test_extract_columns():
    with TemporaryDirectory() as tempdir:
        test_file_path = os.path.join(tempdir, "model.yml")
        with open(test_file_path, "w") as f:
            f.write(sample_yaml_content)
        columns = extract_columns(test_file_path)
        assert len(columns) == 1
        assert columns[0][0] == "computerId"
        assert columns[0][1] == "int"


def test_extract_relationships():
    with TemporaryDirectory() as tempdir:
        test_file_path = os.path.join(tempdir, "model.yml")
        with open(test_file_path, "w") as f:
            f.write(sample_yaml_content)
        relationships = extract_relationships(test_file_path)
        assert len(relationships) == 1
        assert relationships[0][0] == "files"
        assert relationships[0][1] == "computer_files"
        assert relationships[0][3] == "computerId"


def test_generate_dbml():
    with TemporaryDirectory() as tempdir:
        test_file_path = os.path.join(tempdir, "model.yml")
        with open(test_file_path, "w") as f:
            f.write(sample_yaml_content)
        dbml_output = generate_dbml(test_file_path)
        assert isinstance(dbml_output, str)
        assert "Table model" in dbml_output
        assert "computerId int [ref: - computer_files.computerId]" in dbml_output



"""src"""
import os
from functools import reduce
from typing import List, Tuple, Dict, Union
import yaml
from jinja2 import Template


def load_yml(file: str) -> Dict:
    """load_yml"""
    with open(file, "r") as stream:
        try:
            return yaml.safe_load(stream)
        except yaml.YAMLError as err:
            print(err)


def extract_columns(
    file: str, include_non_join_keys: bool = False
) -> List[Tuple[str, str]]:
    """extract_columns"""
    data = load_yml(file)
    columns = data["models"][0]["columns"]
    relationships = data["models"][0].get("relationships", [])
    join_cols = [rel["join"][0]["local"] for rel in relationships]

    if include_non_join_keys or not relationships:
        return [(col["name"], "int") for col in columns]
    else:
        return [
            (col["name"], "int")  # Always int
            for col in columns
            if col["name"] in join_cols
            or "Id" in col["name"]
            or "id" in col["name"]  # Include `id` & `ID` columns always
        ]


def extract_relationships(file: str) -> List[Tuple[str, str, str, str, str]]:
    """extract_relationships"""
    data = load_yml(file)
    relationships = data["models"][0].get("relationships", [])
    return [
        (
            rel["name"],
            rel["table"],
            rel["type"],
            rel["join"][0]["local"],
            rel["join"][0]["remote"],
        )
        for rel in relationships
    ]


def generate_dbml(file: str, include_non_join_keys: bool = False) -> str:
    """generate_dbml"""
    table_name = os.path.splitext(os.path.basename(file))[0]
    columns = extract_columns(file, include_non_join_keys)
    relationships = extract_relationships(file)

    columns_with_refs = []
    for name, _ in columns:
        column: Dict[str, Union[str, bool]] = {"name": name, "is_ref": False}
        for _, table, _, local, remote in relationships:
            if name == local:
                column.update(
                    {"is_ref": True, "ref_table": table, "ref_column": remote}
                )
        columns_with_refs.append(column)

    models = [
        {
            "name": table_name,
            "columns": columns_with_refs,
        }
    ]

    dbml_template = Template(
        """
    {% for model in models %}
    Table {{ model.name }} {
        {% for column in model.columns %}
        {{ column.name }} int{% if column.is_ref %} [ref: - {{ column.ref_table }}.{{ column.ref_column }}]{% endif %}
        {% endfor %}
    }
    {% endfor %}
    """
    )

    return dbml_template.render(models=models)


def generate_dbml_schema(directory: str, include_non_join_keys: bool = False) -> str:
    """generate_dbml_schema"""
    yml_files = [
        os.path.join(directory, file)
        for file in os.listdir(directory)
        if file.endswith(".yml")
    ]
    dbml_files = map(lambda file: generate_dbml(file, include_non_join_keys), yml_files)
    return reduce(lambda a, b: a + b, dbml_files, "")



"""cli"""
import click
from erdgen.src.erdgen import generate_dbml_schema


@click.command()
@click.option("--directory", default=".", help="Directory to search for yml files")
@click.option(
    "--include_non_join_keys", default=False, help="Include non join keys", type=bool
)
def main(directory: str, include_non_join_keys: bool) -> None:
    """main"""
    print(generate_dbml_schema(directory, include_non_join_keys))



#!/usr/bin/env python

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    author="Andrew Moss",
    author_email="andrew.moss@neofinancial.com",
    python_requires=">=3.6",
    name="erdgen",
    version="0.0.1",
    description="Generate a DBML ERD from DBT YML relationships",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
    license="MIT license",
    packages=find_packages(include=["erdgen", "erdgen.*"]),
    url="https://github.com/neo-andrew-moss/erdgen",
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["click", "PyYAML", "Jinja2", "typing_extensions"],
    tests_require=["pytest"],
)
